/* test program for energy-loss functions of MOCADI */
/* Can be used also as standalone simple ATIMA      */
/* version adjusted for mocadi 4-2, 01.02.2018      */
/* from the full MOCADI by Naohito Iwasa, Tohoku U. */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#define TBLMAX 256
#include "mocadi.h"
#include "atima.h"
#include "yesno.c"

char MOCADI_DATA[256],MOCADI_SPLINES[256],MOCADI_SPLINES_GZ[256],filename[256];
struct splines atima_spline[120][I_SAVE_MAX+1];
void frsmode(double *d1,double *d2,double *m2,double *d3,double *m3);
int b_check=0;
int b_mass_mode=0;
int i_save,i_asave[I_SAVE_MAX+1],i_zsave[I_SAVE_MAX+1],i_fsave[I_SAVE_MAX+1];
double d_asave[I_SAVE_MAX+1];
FILE *out;

main()
{
  double amu=931.494092;
  double a=238.0508,d_at=63.546,e1=1000.;
  int z=92,i_zt=29,i_at,i;
  double e2,r1,sr1,r2,sr2,sa,se,d_rho,sr,dedx,qmean1,qmean2; 
  double t0=1000;

  if(getenv("MOCADI_SPLINES")!=NULL)
    sprintf(MOCADI_SPLINES,"%s/\0",getenv("MOCADI_SPLINES"));
  else if(getenv("MOCADI_DIR")!=NULL)
    sprintf(MOCADI_SPLINES,"%s/splines/\0",getenv("MOCADI_DIR"));
  else{
    strcpy(MOCADI_SPLINES, MOCADI_HOME);
    strcat(MOCADI_SPLINES,"/splines/");
  }
  if(getenv("MOCADI_SPLINES_GZ")!=NULL)
    sprintf(MOCADI_SPLINES_GZ,"%s/\0",getenv("MOCADI_SPLINES_GZ"));
  else if(getenv("MOCADI_DIR")!=NULL)
    sprintf(MOCADI_SPLINES_GZ,"%s/splines_gz_1.41/\0",getenv("MOCADI_DIR"));
  else{
    strcpy(MOCADI_SPLINES_GZ, MOCADI_HOME);
    strcat(MOCADI_SPLINES_GZ,"/splines_gz_1.41/");
  }

  out=stdout; 
  puts("***************************************************************");
  puts("**  ATIMA calculation V1.41 based on precalculated tables    **");
  puts("**  export MOCADI_SPLINES_GZ='/u/weick/mocadi/splines_gz_1.41' ");
  puts("**  selected compounds for input Zp = 201-220.               **");
  puts("**  Sorry, dE/dx output only correct for first material in   **");
  puts("**  sequence of many inputs, then restart program, rest ok.  **");
  puts("***************************************************************");
  
 l0:
  printf("\n***** input following parameters *****\n\n");
  z =askdouble("nuclear charge of projectile ",z);
  a =askdouble("mass of projectile [amu]     ",a);
  e1=askdouble("energy of fragment [MeV/u]   ",e1);
  i_zt=askint( "atomic number of material    ",i_zt);
  if(i_zt<200) { 
    d_at=askdouble( "molar mass of material [amu] ",d_at);
  }
  else {
    d_at = 0;  /* compound has fixed molar mass*/
    printf("molar mass of material = default\n");
  }
  t0 =askdouble("material thickness [mg/cm2]  ",t0);

  i_at = rint(d_at);
  if(i_zt<=0||i_at<0)return;
  
  // in case At=0 use std. value from spline file
  if (i_at==0) {
    i_at = 2 * i_zt ; // dummy value
    sirera2(i_at,i_zt);
    d_at = d_asave[1];
  }
  else sirera2(i_at,i_zt);

  sisigra1(a,z,e1,d_at,i_zt,&r1,&sr1);
  sienver1(a,z,0,e1,d_at,i_zt,t0,&e2,0.,&i);
  sisigra1(a,z,e2,d_at,i_zt,&r2,&sr2);
  sa=sisigan(a,z,e1,e2,d_at,i_zt);
  se=sisigen(a,z,e1,e2,d_at,i_zt);

  //dedx uses atima splines directly and must be rescaled here
  dedx=atima_dedx(&atima_spline[z-1][1],e1)*d_asave[1]/d_at;
  // fixed index [1] works only for first matter, i_save needed for more,
  // otherwise scaling with wrong d_asave -> wrong dE/dx, rest is ok.

  // use spline for mean projectile charge
  qmean1=atima_qmean(&atima_spline[z-1][1],e1); 
  qmean2=atima_qmean(&atima_spline[z-1][1],e2); 
  if (e1<10) qmean1 = 0 ;
  if (e2<10) qmean2 = 0. ;      
  
  printf("\n projectile Z=%d A=%lf E=%lf MeV/u \n",z,a,e1);
  printf(" target     Z=%d A=%f t=%lf\n",i_zt,d_at,t0);
  printf(" dE/dx at entrance : %lf MeV/(mg/cm2)\n",dedx);
  printf(" exit energy       : %lf MeV/u\n",e2);
  printf(" energy straggling : %lf MeV/u (std.dev.)\n",se/a);
  printf(" range             : %lf mg/cm2 -> %lf mg/cm2\n",r1,r2);
  printf(" range straggling  : %lf mg/cm2 (std.dev.)\n",sr1);
  printf(" angular straggling: %lf mrad (std.dev.)\n",sa*1000);
  printf(" <q>_p in -> out   : %lf -> %lf e\n",qmean1,qmean2); 

  goto l0;
  
}
