/* test program for energy-loss functions of MOCADI       */
/* from the full MOCADI by Naohito Iwasa, Tohoku Univers. */
/* Can be used also as standalone simple ATIMA            */
/* version adjusted for mocadi 4-2, 01.02.2018            */
/* start with: ./atimawww 92 238.01 1000. 29 63.546 100.  */

#include <stdio.h> 
#include <stdlib.h>
#include <math.h>
#include <string.h>
#define TBLMAX 256
#include "mocadi.h"
#include "atima.h"
#include "yesno.c" 

char MOCADI_DATA[256],MOCADI_SPLINES[256],MOCADI_SPLINES_GZ[256],filename[256];
struct splines atima_spline[120][I_SAVE_MAX+1];
void frsmode(double *d1,double *d2,double *m2,double *d3,double *m3);
int b_check=0;
int b_mass_mode=0;
int i_save,i_asave[I_SAVE_MAX+1],i_zsave[I_SAVE_MAX+1],i_fsave[I_SAVE_MAX+1];
double d_asave[I_SAVE_MAX+1];
FILE *out;

main(int argc, char *argv[])
{
  double amu=931.494092;
  double a=238.0011,d_at=63.546,e1=1000.;
  int z=92,i_zt=29,i_at,i;
  double e2,r1,sr1,r2,sr2,sa,se,d_rho,sr,dedx,qmean1,qmean2; // HWE
  double t0=1000;

  strcpy(MOCADI_SPLINES,"/WWW/weick/atima");
  strcat(MOCADI_SPLINES,"/splines_1.41/");

  strcpy(MOCADI_SPLINES_GZ,"/WWW/weick/atima");
  strcat(MOCADI_SPLINES_GZ,"/splines_gz_1.41/");

  if((argc<7)||(strncmp(argv[1],"--help",1)==0)) {
    printf("  usage  : atimawww-1.41 Zp Ap[amu] E[MeV/u] Zt At[g/mol] t[mg/cm^2]\n");
    printf("  example: atimawww-1.41 92 238.001 1000. 13 26.9985 100.\n");
    exit(1) ;
  }
  sscanf(argv[1],"%3d",&z);
  sscanf(argv[2],"%lf",&a);
  sscanf(argv[3],"%lf",&e1);
  sscanf(argv[4],"%3d",&i_zt);
  sscanf(argv[5],"%lf",&d_at);
  sscanf(argv[6],"%lf",&t0);
  
  i_at = rint(d_at);
  if(i_zt<=0||i_at<0)return;
  
  // in case At=0 use std. value from spline file
  if (i_at==0) {
    i_at = 2 * i_zt ; // dummy value
    sirera2(i_at,i_zt);
    d_at = d_asave[1];
  }
  else sirera2(i_at,i_zt);

  sisigra1(a,z,e1,d_at,i_zt,&r1,&sr1);
  sienver1(a,z,0,e1,d_at,i_zt,t0,&e2,0.,&i);
  sisigra1(a,z,e2,d_at,i_zt,&r2,&sr2);
  sa=sisigan(a,z,e1,e2,d_at,i_zt);
  se=sisigen(a,z,e1,e2,d_at,i_zt);
  //dedx uses atima splines directly and must be rescaled here
  dedx=atima_dedx(&atima_spline[z-1][1],e1)*d_asave[1]/d_at;
  // use spline for mean projectile charge
  qmean1=atima_qmean(&atima_spline[z-1][1],e1); 
  qmean2=atima_qmean(&atima_spline[z-1][1],e2); 
  if (e1<10) qmean1 = 0. ;
  if (e2<10) qmean2 = 0. ;  
  
  printf("\n");
  printf("<br> Atima 1.41 <br>\n");
  printf(" projectile        : Z=%d A=%lf E=%lf MeV/u <br>\n",z,a,e1);
  printf(" target            : Z=%d A=%f thickness=%lf mg/cm^2<br>\n",i_zt,d_at,t0);
  printf(" dE/dx at entrance : %lf MeV/(mg/cm^2)<br>\n",dedx);
  printf(" exit energy       : %lf MeV/u <br>\n",e2);
  printf(" energy straggling : %lf MeV/u (std.dev.)<br>\n",se/a);
  printf(" range             : %lf mg/cm^2 <br>\n",r1);
  printf(" range straggling  : %lf mg/cm^2(std.dev.)<br>\n",sr1);
  printf(" angular straggling: %lf mrad (std.dev.)<br>\n",sa*1000);
  printf(" q_p in - out      : %lf - %lf <br>\n",qmean1,qmean2); 
  printf("done\n");
  fflush(stdout); 
   
}
