#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "atima.h"

// prototyping for old ATIMA functions in FORTRAN,
// obsolete, replaced by new bvalue2() and e_out2() in C, HWE
//float bvalue_(float *, float *, int *, int *, float *, int *);
//float e_out_(float *, float *, int *, int *, float *, float *, float *);

void read_atima_spline(FILE *fp, struct bspline *pspline){
   int i;   
   float temp;

   fscanf(fp, "%d%d", &pspline->n, &pspline->k);
   pspline->t = (float*) malloc( sizeof(float) * (pspline->n + pspline->k) );
   pspline->b = (float*) malloc( sizeof(float) * pspline->n );
   for ( i= 0; i < pspline->n + pspline->k;i++) { 
     fscanf(fp, "%e",&temp);
     pspline->t[i] = temp; 
   }
   for ( i= 0; i < pspline->n; i++){
     fscanf(fp, "%e",&temp);
     pspline->b[i] = temp;
   }
}

int get_atima_spline(const char * file, struct splines *s) {
   int i;
   FILE *fp;

   if ((fp=fopen(file,"r")) == NULL) {
     printf("cannot open file %s \n", file);
     return(-1);
   }

   fscanf(fp, "%d%e", &s->z_projectile, &s->m_projectile);
   fscanf(fp, " %d", &s->n_nuc);
   s->z_target = (int *) malloc(sizeof(int) * (s->n_nuc));
   s->count = (float *) malloc(sizeof(float) * (s->n_nuc));
   s->pot = (float *) malloc(sizeof(float) * (s->n_nuc));
   s->m_target = (float *) malloc(sizeof(float) * (s->n_nuc));   
   for( i=0; i< (s->n_nuc); i++ ){
      fscanf(fp, "%d%e%e%e", &s->z_target[i], &s->count[i], &s->pot[i], &s->m_target[i]);  
   } 
   fscanf(fp, "%e%d%e", &s->rho, &s->igas, &s->fntp);
   s->range_spline   = (struct bspline *) malloc(sizeof(struct bspline));
   s->rstragg_spline = (struct bspline *) malloc(sizeof(struct bspline));  
   s->astragg_spline = (struct bspline *) malloc(sizeof(struct bspline));
   s->qmean_spline   = (struct bspline *) malloc(sizeof(struct bspline)); // HWE
   read_atima_spline(fp, s->range_spline);
   read_atima_spline(fp, s->rstragg_spline);
   read_atima_spline(fp, s->astragg_spline);
   read_atima_spline(fp, s->qmean_spline); // HWE
   
   fclose(fp);
   return(0);
}
/*************************************
 *                                   *
 * read ATIMA splines gzipped     *
 *                                   *
 *************************************/

int get_atima_spline_gzipped(const char * file, struct splines *s) {
   int i;
   FILE *fp;
   char command[266];

   sprintf(command,"gunzip -c %s",file);
   if ((fp=popen(command,"r")) == NULL) {
     printf("cannot open file %s \n", file);
     return(-1);
   }

   fscanf(fp, "%d%e", &s->z_projectile, &s->m_projectile);
   fscanf(fp, " %d", &s->n_nuc);
   s->z_target = (int *) malloc(sizeof(int) * (s->n_nuc));
   s->count = (float *) malloc(sizeof(float) * (s->n_nuc));
   s->pot = (float *) malloc(sizeof(float) * (s->n_nuc));
   s->m_target = (float *) malloc(sizeof(float) * (s->n_nuc));   
   for( i=0; i< (s->n_nuc); i++ ){
      fscanf(fp, "%d%e%e%e", &s->z_target[i], &s->count[i], &s->pot[i], &s->m_target[i]);  
   } 
   fscanf(fp, "%e%d%e", &s->rho, &s->igas, &s->fntp);
   s->range_spline = (struct bspline *) malloc(sizeof(struct bspline));
   s->rstragg_spline = (struct bspline *) malloc(sizeof(struct bspline));  
   s->astragg_spline = (struct bspline *) malloc(sizeof(struct bspline));
   s->qmean_spline = (struct bspline *) malloc(sizeof(struct bspline)); 
   read_atima_spline(fp, s->range_spline);
   read_atima_spline(fp, s->rstragg_spline);
   read_atima_spline(fp, s->astragg_spline);
   read_atima_spline(fp, s->qmean_spline); 

   pclose(fp);
   return(0);
}

/**************************************
 *                                    *
 * free memory for ATIMA splines      *
 *                                    *
 **************************************/

void free_atima_spline(struct splines *s) { 
   free(s->z_target);
   free(s->count);
   free(s->pot);
   free(s->m_target);
   free(s->range_spline->t);
   free(s->range_spline->b);
   free(s->range_spline);
   free(s->rstragg_spline->t);
   free(s->rstragg_spline->b);
   free(s->rstragg_spline);
   free(s->astragg_spline->t);
   free(s->astragg_spline->b);
   free(s->astragg_spline);
   free(s->qmean_spline->t); 
   free(s->qmean_spline->b); 
   free(s->qmean_spline);    
}

float atima_range(struct splines *s, float energy) {
   int i = 0;
   int n, k ;
   
   //return bvalue_(s->range_spline->t,s->range_spline->b,&(s->range_spline->n),&(s->range_spline->k),&energy,&i);
   return bvalue2(s->range_spline,energy,i);
}

float atima_enver(struct splines *s, float energy, float thickness) {
   float range;

   range = atima_range(s, energy);
   if ( range <= thickness ) return 0.0;
   
   //return e_out_(s->range_spline->t,s->range_spline->b,&(s->range_spline->n),&(s->range_spline->k),&range,&energy,&thickness);
   return e_out2(s->range_spline,range,energy,thickness);
}

float atima_sigra2(struct splines *s, float energy) {
    int i = 0;
    float r1;

    r1 = bvalue2(s->rstragg_spline,energy,i);
    //r1 = bvalue_(s->rstragg_spline->t,s->rstragg_spline->b,&(s->rstragg_spline->n),&(s->rstragg_spline->k),&energy,&i);

    return (float) (double)r1;
}

float atima_dedx(struct splines *s, float energy) { 
    int i = 1;
 
    // return s->m_projectile / bvalue_(s->range_spline->t,s->range_spline->b,&(s->range_spline->n),&(s->range_spline->k),&energy,&i);
    return s->m_projectile / bvalue2(s->range_spline,energy,i);   
}


float atima_domega2de(struct splines *s, float energy) {
    int i = 1;
    float do2de;
    
    //do2de = bvalue_(s->rstragg_spline->t,s->rstragg_spline->b,&(s->rstragg_spline->n),&(s->rstragg_spline->k),&energy,&i);
    do2de = bvalue2(s->rstragg_spline,energy,i);

    return (float) do2de ;
}


float atima_da2de(struct splines *s, float energy) { 
    int i = 1;
    float da2de;
    
    //da2de = bvalue_(s->astragg_spline->t,s->astragg_spline->b,&(s->astragg_spline->n),&(s->astragg_spline->k),&energy,&i);
    da2de = bvalue2(s->astragg_spline,energy,i);
	
    return (float) da2de ;   
}


float atima_astragg(struct splines *s, float energy_in, float energy_out) {
    int i = 0;
    float a1;
    float a2;
             
    //a1 = bvalue_(s->astragg_spline->t,s->astragg_spline->b,&(s->astragg_spline->n),&(s->astragg_spline->k),&energy_in,&i);
    //a2 = bvalue_(s->astragg_spline->t,s->astragg_spline->b,&(s->astragg_spline->n),&(s->astragg_spline->k),&energy_out,&i);
    a1 = bvalue2(s->astragg_spline,energy_in ,i);
    a2 = bvalue2(s->astragg_spline,energy_out,i);
    
    if(a1<a2){
      printf("*** warning variance of ang.scatt<0, corrected to 0.\n");
      return((float) 0);
    }
    return ((float) sqrt( (double) (a1 - a2) ));                                
}


// function for q-mean of projectile, HWE 31.01.2018
float atima_qmean(struct splines *s, float energy) { 
    int i = 1;
    float qmean;
    
    //qmean = bvalue_(s->qmean_spline->t,s->qmean_spline->b,&(s->qmean_spline->n),&(s->qmean_spline->k),&energy,&i);
    qmean = bvalue2(s->qmean_spline,energy,i);
	
    return (float) qmean ;   
}
