// based on function ZBRENT copied from Numerical Recipes, modified for atima
// converted from FORTRAN f_splines.f to ANSI-C, H. Weick 22.Feb.2018

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "atima.h"

#define ITMAX 100  // Maximum allowed number of iterations.
#define EPS 3.0e-8 // Machine floating-point precision.
#define TOL 1.E-6  // fixed tolerances
#define kmax 20    // max. in bvalue()


//  calculate exit energy after target from range spline
float e_out2(struct bspline *r, double range,double ei,double thickness)
{
  double A,B,C,D,E,FA,FB,FC,TOL1,XM,P,Q,R,S,sign ;
  int ITER ;
  int n,k ;
  
      n = r->n ; 
      k = r->k ;

      A=ei;
      B=0.;
      FA= range - bvalue2(r,A,0) - thickness ;
      FB= range - bvalue2(r,B,0) - thickness ;
      if(FB*FA > 0.) printf("Root must be bracketed for ZBRENT.\n");
      FC=FB;
      for(ITER=1; ITER<=ITMAX; ITER++) {
        if(FB*FC > 0.) {
          C=A;
          FC=FA;
          D=B-A;
          E=D;
        }
        if(fabs(FC) < fabs(FB)) {
          A=B;
          B=C;
          C=A;
          FA=FB;
          FB=FC;
          FC=FA;
        }
        TOL1=2.*EPS*fabs(B)+0.5*TOL;
        XM=.5*(C-B);
        if(fabs(XM) <= TOL1  ||  FB == 0.){
          return B; // E_out = B
        }
        if(fabs(E) >= TOL1  &&  fabs(FA) > fabs(FB)) {
          S=FB/FA;
          if(A == C) {
            P=2.*XM*S;
            Q=1.-S;
          }
          else {
            Q=FA/FC;
            R=FB/FC;
            P=S*(2.*XM*Q*(Q-R)-(B-A)*(R-1.));
            Q=(Q-1.)*(R-1.)*(S-1.);
          }
          if(P > 0.) Q=-Q;
          P=fabs(P);
          if(2.*P < (3.*XM*Q-fabs(TOL1*Q) < fabs(E*Q) ? 3.*XM*Q-fabs(TOL1*Q) : fabs(E*Q))) {
            E=D;
            D=P/Q;
          }
          else {
            D=XM;
            E=D;
          }
        }
        else {
          D=XM;
          E=D;
        }
        A=B;
        FA=FB;
        if(fabs(D) > TOL1) {
          B=B+D;
        }
        else {	  
          //  B=B+SIGN(TOL1,XM)  replaced, no SIGN() in standard C
          if (XM < 0) sign = -1;
          else sign =  1;
          B += sign*TOL1 ;
        }
        FB= range - bvalue2(r,B,0) - thickness ;
      } 
      printf("ZBRENT exceeding maximum iterations.\n");
      return B ;
}

/*-------------------------------------------------------------------*/
//  from  a practical guide to splines by C. de Boor, calls  interv()

//  calculates value at  x  of  jderiv-th derivative of spline from b-repr.
//  the spline is taken to be continuous from the right, EXCEPT at the
//  rightmost knot, where it is taken to be continuous from the left.

//******  i n p u t ******
//  t, bcoef, n, k......forms the b-representation of the spline  f  to
//        be evaluated. specifically,
//  t.....knot sequence, of length  n+k, assumed nondecreasing.
//  bcoef.....b-coefficient sequence, of length  n .
//  n.....length of  bcoef  and dimension of spline(k,t),
//        a s s u m e d  positive .
//  k.....order of the spline .

//  w a r n i n g . . .   the restriction  k .le. kmax (=20)  is imposed
//        arbitrarily by the dimension statement for  aj, dl, dr  below,
//        but is  n o w h e r e  c h e c k e d  for.

//  x.....the point at which to evaluate .
//  jderiv.....integer giving the order of the derivative to be evaluated
//        a s s u m e d  to be zero or positive.

//******  o u t p u t  ******
//  bvalue.....the value of the (jderiv)-th derivative of  f  at  x .

//******  m e t h o d  ******
//     The nontrivial knot interval  (t(i),t(i+1))  containing  x  is lo-
//  cated with the aid of  interv . The  k  b-coeffs of  f  relevant for
//  this interval are then obtained from  bcoef (or taken to be zero if
//  not explicitly available) and are then differenced  jderiv  times to
//  obtain the b-coeffs of  (d**jderiv)f  relevant for that interval.
//  Precisely, with  j = jderiv, we have from x.(12) of the text that

//     (d**j)f  =  sum ( bcoef(.,j)*b(.,k-j,t) )

//  where
//                   / bcoef(.),                     ,  j .eq. 0
//                   /
//    bcoef(.,j)  =  / bcoef(.,j-1) - bcoef(.-1,j-1)
//                   / ----------------------------- ,  j .gt. 0
//                   /    (t(.+k-j) - t(.))/(k-j)

//     Then, we use repeatedly the fact that

//    sum ( a(.)*b(.,m,t)(x) )  =  sum ( a(.,x)*b(.,m-1,t)(x) )
//  with
//                 (x - t(.))*a(.) + (t(.+m-1) - x)*a(.-1)
//    a(.,x)  =    ---------------------------------------
//                 (x - t(.))      + (t(.+m-1) - x)

//  to write  (d**j)f(x)  eventually as a linear combination of b-splines
//  of order  1 , and the coefficient for  b(i,1,t)(x)  must then be the
//  desired number  (d**j)f(x). (see x.(17)-(19) of text).


float bvalue2(struct bspline *r, float x, int jderiv)
{
  int i,ilo,imk,j,jc,jcmin,jcmax,jj,kmj,km1,mflag,nmi,jdrvp1;
  float aj[kmax],dl[kmax],dr[kmax],fkmj;
  float bvalue1 ;
  int n, k;

  // struct *r can be any of the four ATIMA splines
  // indices in spline array shifted by -1 in translatiomn from FORTRAN
  n = r->n ;
  k = r->k ;
  bvalue1 = 0.;
  if (jderiv >= k)   goto L99;

//  *** Find  i   s.t.   1 .le. i .lt. n+k   and   t(i) .lt. t(i+1)   and
//      t(i) .le. x .lt. t(i+1) . If no such i can be found,  x  lies
//      outside the support of  the spline  f , hence  bvalue = 0.
//      (The asymmetry in this choice of  i  makes  f  rightcontinuous, except
//      at  t(n+k) where it is leftcontinuous.)
      interv2( r, n+k, x, &i, &mflag);
      if (mflag  !=  0)   goto L99;
//  *** if k = 1 (and jderiv = 0), bvalue = bcoef(i).
      km1 = k - 1;
      if (km1  >  0)      goto L1;
      bvalue1 = r->b[i-1];   
                          goto L99;

//  *** store the k b-spline coefficients relevant for the knot interval
//     (t(i),t(i+1)) in aj(1),...,aj(k) and compute dl(j) = x - t(i+1-j),
//     dr(j) = t(i+j) - x, j=1,...,k-1 . set any of the aj not obtainable
//     from input to zero. set any t.s not obtainable equal to t(1) or
//     to t(n+k) appropriately.
L1:   jcmin = 1;
      imk = i - k;
      if (imk  >=  0) goto L8;
      jcmin = 1 - imk;
      for(j=1; j<=i; j++) {
L5:      dl[j] = x - r->t[i-j]; 
      }
      for(j=i; j<=km1; j++) {
         aj[k-j] = 0.;
L6:      dl[j] = dl[i];
      }
      goto L10;
L8:   for(j=1; j<=km1; j++) {
      L9:      dl[j] = x - r->t[i-j]; 
      }
L10:  jcmax = k;
      nmi = n - i;
      if (nmi  >=  0)  goto L18;
      jcmax = k + nmi;
      for(j=1; j<=jcmax; j++) {
L15:     dr[j] = r->t[i+j-1] - x; 
      }
      for(j=jcmax; j<=km1; j++) {
         aj[j+1] = 0.;
L16:     dr[j] = dr[jcmax];
      }	 
      goto L20;
L18:  for(j=1; j<=km1; j++) {
L19:     dr[j] = r->t[i+j-1] - x; 
      }
L20:  for(jc=jcmin; jc<=jcmax; jc++) {
L21:     aj[jc] = r->b[imk + jc -1]; 
      }
      
//  *** differentiate the coefficients jderiv times.
      if (jderiv  ==  0)  goto L30;
      for(j=1; j<=jderiv; j++) {
         kmj = k-j;
         fkmj = (float) kmj;
         ilo = kmj;
         for(jj=1; jj<=kmj; jj++) {
            aj[jj] = ((aj[jj+1] - aj[jj])/(dl[ilo] + dr[jj]))*fkmj;
L23:        ilo = ilo - 1;
	 }
      }

//  *** compute value at  x  in (t(i),t(i+1)) of jderiv-th derivative,
//     given its relevant b-spline coeffs in aj(1),...,aj(k-jderiv).
L30:  if (jderiv  ==  km1)  goto L39;
      jdrvp1 = jderiv + 1 ;
      for(j=jdrvp1; j<=km1; j++) {
         kmj = k-j;
         ilo = kmj;
         for(jj=1; jj<=kmj; jj++) {
            aj[jj] = (aj[jj+1]*dl[ilo] + aj[jj]*dr[jj])/(dl[ilo]+dr[jj]);
L33:        ilo = ilo - 1;
	 }
      } 
L39:  bvalue1 = aj[1];

L99:  return bvalue1;
}

//--------------------------------------------------------------------
//  Find right interval on spline
//  The indices in the splines were shifted by -1 in the translation from FORTRAN
//     H. Weick, 22.02.2018
void interv2(struct bspline *r, int lxt, float x, int *left, int *mflag)
{
  int ihi,istep,middle ;
  static int ilo=1 ;
  
      ihi = ilo + 1;     
      if (ihi < lxt)         goto L20;
      if (x >= r->t[lxt-1])  goto L110; 
      if (lxt <=  1)         goto L90;
      ilo = lxt - 1;
      ihi = lxt;

 L20:  if (x >= r->t[ihi-1]) goto L40; 
      if (x >= r->t[ilo-1])  goto L100;

//   **** now x .lt. xt(ilo) . decrease  ilo  to capture  x .
      istep = 1;
L31:  ihi = ilo;
      ilo = ihi - istep;
      if (ilo <= 1)          goto L35;
      if (x >= r->t[ilo-1])  goto L50; 
      istep = istep*2;
                             goto L31;
L35:  ilo = 1;
      if (x  <  r->t[0])     goto L90; 
                             goto L50;
//    **** now x .ge. xt(ihi) . increase  ihi  to capture  x .
L40:  istep = 1;
L41:  ilo = ihi;
      ihi = ilo + istep;
      if (ihi  >=  lxt)      goto L45;
      if (x  <  r->t[ihi-1]) goto L50; 
      istep = istep*2;
                             goto L41;
L45:  if (x >= r->t[lxt-1])  goto L110;
      ihi = lxt;

//    **** now xt(ilo) .le. x .lt. xt(ihi) . narrow the interval.
L50:  middle = (ilo + ihi)/2;
      if (middle == ilo)      goto L100;
//    note, it is assumed that middle = ilo in case ihi = ilo+1 .
      if (x < r->t[middle-1]) goto L53; 
      ilo = middle;
                              goto L50;
L53:  ihi = middle;
                              goto L50;
//    **** set output and return.
L90:  *mflag = -1;
      *left = 1;
      return;
L100: *mflag = 0;
      *left = ilo;
      return;
L110: *mflag = 1;
      if (x  ==  r->t[lxt-1]) *mflag = 0; 
      *left = lxt;
L111: if (*left  ==  1)       return;
      *left = *left - 1;
      if (r->t[*left-1]  <  r->t[lxt-1]) return; 
                              goto L111;
  
  return;
}

